"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const http = require("http");
const os = require("os");
require("reflect-metadata");
const util_1 = require("util");
const Error_1 = require("./Error");
const Fork_1 = require("./Fork");
const kafka_1 = require("./kafka");
const types_1 = require("./types");
const Worker_1 = require("./Worker");
class Microservice extends kafka_1.default {
    /**
     * Contructor de la clase
     * @param conf Configuracion del servicio
     */
    constructor(conf) {
        // Llamar el constructor del padre
        super(conf);
        /**
         * Coleccion de errores
         * @private
         * @type Map
         */
        this.ErrorColl = new Map();
        /**
         * Hostname del nodo
         * @private
         */
        this.hostname = os.hostname();
        // Errores por default
        const errDefault = {
            '-99': 'Ocurrio una excepcion, favor de intentar de nuevo.',
            '0': 'Smoke Test',
            '1': 'Este servicio no puede ser consumido de forma externa',
        };
        // Errores
        let err;
        // Ver el tipo de aplicacion
        if (this.conf.Type === types_1.TypeService.Fork) {
            // Crear la instancia para el Fork
            this.app = new Fork_1.default(this.conf);
        }
        else if (this.conf.Type === types_1.TypeService.Worker) {
            // Crear la instancia de Worker
            this.app = new Worker_1.default(this.conf);
        }
        else {
            this.logs.error(`El tipo ${this.conf.Type} no es un elemento reconocido`);
            process.exit(-1); // Salir del proceso
        }
        // Revisar si esta definido los errores.
        if (Reflect.hasMetadata('definition:errors', this)) {
            // Recuperar la funcion
            const fnc = this.get(Reflect.getMetadata('definition:errors', this).key);
            // Asignar
            err = Object.assign(fnc(), errDefault);
        }
        else {
            // Asignar
            err = errDefault;
        }
        // Almacenar
        for (const key of Object.keys(err)) {
            // Almacenar el Error
            this.ErrorColl.set(parseInt(key, 0), err[key]);
        }
    }
    /**
     * Metodo para el procesamiento de mensajes
     * @param data mensaje
     */
    message(data) {
        return __awaiter(this, void 0, void 0, function* () {
            // Metodo que se ejecutara
            let methondExecute = null;
            // Validar si no es una peticion de SmokeTest
            if ('smoketest' in data.data) {
                this.logs.info('Ejecutando las pruebas de smoketest');
                // Ejecutar la prueba
                try {
                    if (yield this.smoketest()) {
                        // Retornar el valor originar
                        this.response(data, 0, true);
                    }
                    else {
                        // Pasar a enviar el error
                        this.response(data, -1, true);
                    }
                }
                catch (error) {
                    // Generar el transactionId
                    const transactionId = (new Date().getTime());
                    // Indicar el error
                    this.sendError(this.Topic, transactionId, error);
                    // Pasar a enviar el error
                    this.response(data, -1, true);
                }
            }
            else {
                // Ver si esta activo el modo Debug
                if (this.conf.Debug) {
                    // Imprimir el contenido
                    this.logs.info(`
                    ENTRADA [${this.Topic}]:
                    HEADERS: [${JSON.stringify(data.headers)}]
                    DATA: [${JSON.stringify(data.data)}]
                `);
                }
                // Validar si es callback y no publico
                if (data.metadata.callback === this.Topic && !this.conf.Public) {
                    // Indicar que no se puede acceder por no ser publico
                    this.logs.warning('No se permite el acceso, no es publico.');
                    // Regresar los datos originales
                    return this.response(data, 1);
                }
                else if (data.metadata.time.indexOf('Z') > -1 && this.conf.ClearQueque) {
                    // Cast de la fecha
                    const date = new Date(data.metadata.time);
                    // Fecha actual
                    const currDate = new Date();
                    // Sacar la diferencia de fechas
                    const diff = Math.round((currDate.getTime() - date.getTime()) / 60000);
                    // Si pasaron 5 minutos ignorar el mensaje
                    if (diff >= 5) {
                        return;
                    }
                }
                // Validar que cuente con el uuid y method
                if (data.metadata.method && data.metadata.uuid) {
                    if (data.metadata.method === types_1.HttpMethod.PUT) {
                        // Buscar si cuenta con una definicion de actualizacion
                        methondExecute = this.getConfHandler('definition:update');
                    }
                    else if (data.metadata.method === types_1.HttpMethod.DELETE) {
                        // Buscar el metodo de eliminacion
                        methondExecute = this.getConfHandler('definition:delete');
                    }
                    else if (data.metadata.method === types_1.HttpMethod.GET) {
                        // Buscar el metodo de consulta
                        methondExecute = this.getConfHandler('definition:get');
                    }
                }
                else if (data.metadata.method && data.metadata.method === types_1.HttpMethod.GET) {
                    // Buscar el metodo de List
                    methondExecute = this.getConfHandler('definition:list');
                }
                else if (data.metadata.method && data.metadata.method === types_1.HttpMethod.POST) {
                    // Buscar si cuenta con una definicion de registro
                    methondExecute = this.getConfHandler('definition:create');
                }
                else {
                    // Buscar le metodo de listener
                    methondExecute = this.getConfHandler('');
                }
                // Verficar si no ocurrio un error al consultar la funcion
                if (util_1.isNull(methondExecute)) {
                    return 0;
                }
                // Pasar el mensaje para ser procesado
                try {
                    // Ejecutar
                    const resp = yield this.app.message(data, this.get(methondExecute.key).bind(this));
                    // Enviar la respuesta del error
                    this.response(resp.response, resp.errorCode);
                }
                catch (error) {
                    // Enviar indicando que ocurrio un error.
                    this.response(error.response, error.errorCode || -99);
                }
            }
        });
    }
    /**
     * Metodo para publicar un error en la cola de mensajes
     * @param service Nombre del servicio
     * @param transactionId Id de la transaccion
     * @param error Error
     */
    sendError(service, transactionId, error) {
        // Asignar el objeto de respuesta
        const message = Buffer.from(JSON.stringify({
            error,
            id: transactionId,
            servicio: service,
        })).toString('base64');
        // Enviar el error
        this.producer.send([{
                key: transactionId,
                messages: [message],
                topic: 'Errores_criticos',
            }], (err) => {
            if (err) {
                // Imprimir el error
                this.logs.error(err);
            }
        });
    }
    /**
     * Metodo para publicar la respuesta
     * @param data Datos de respuesta
     * @param errorCode Codigo de error
     */
    response(data, errorCode, smoketest = false) {
        // Objecto de salida
        let Resp = {};
        // Datos de respuesta
        const dataResponse = data;
        // Objecto de error
        const errorResponse = {
            data: {
                response: {
                    errorCode,
                    userMessage: this.getError(errorCode),
                },
            },
            meta: {
                id_transaction: dataResponse.metadata.id_transaction,
                status: errorCode < 0 ? 'ERROR' : 'SUCCESS',
            },
        };
        // Validar si ocurrio un error
        if (errorCode < 0) {
            // Asignar el hostname
            errorResponse.data.response.hostname = this.hostname;
            // Codigo de error
            errorResponse.data.response.code = errorCode;
            // Eliminar errorcode
            delete errorResponse.data.response.errorCode;
            // Asignar el error
            dataResponse.response = errorResponse;
        }
        // Asignar el tiempo
        dataResponse.metadata.time = (new Date()).toISOString();
        // Indicar el worker
        dataResponse.metadata.worker = data.metadata.owner;
        // Topico
        dataResponse.metadata.owner = this.Topic;
        // Tipo de salida
        dataResponse.metadata.mtype = 'output';
        // Validar que tenga un uowner
        if ('uowner' in data.metadata) {
            // Asignar
            dataResponse.metadata.uworker = data.metadata.uowner;
        }
        // Validar si tiene asignado un uworker
        if ('uworker' in data.metadata) {
            // Asignar
            dataResponse.metadata.uowner = data.metadata.uworker;
        }
        // Topico de respuesta
        let topic = `respuesta_${data.metadata.owner}`;
        // Validar si es una bifurcacion
        if (dataResponse.metadata.bifurcacion) {
            // Asignar el topico
            topic = dataResponse.metadata.callback;
            // Pasar a falso
            dataResponse.metadata.bifurcacion = false;
            // Indicar los datos
            Resp = dataResponse;
        }
        else {
            // Asignar la respuesta
            Resp = {
                _id: dataResponse.metadata.id_transaction,
                metadata: dataResponse.metadata,
                response: dataResponse.response,
            };
        }
        // Revisar si es un smoketest
        if (smoketest) {
            // Objeto de respuesta generica
            Resp.response = {
                data: {
                    response: {
                        code: errorCode,
                        hostname: this.hostname,
                        userMessage: 'Smoke test',
                    },
                },
                meta: {
                    id_transaction: dataResponse.metadata.id_transaction,
                    status: errorCode < 0 ? 'ERROR' : 'SUCCESS',
                },
            };
        }
        // Validar si es modeo DEBUG
        if (this.conf.Debug) {
            // Imprimir por consola lo que se envia
            this.logs.info(`SALIDA [${topic}]: ${JSON.stringify(Resp)}`);
        }
        // Pasar a Base64 la respuesta
        const dataBuffer = Buffer.from(JSON.stringify(Resp)).toString('base64');
        // Enviar la respuesta
        this.producer.send([{
                messages: [dataBuffer],
                topic,
            }], (err) => {
            // Validar si ocurrio un error
            if (err) {
                this.logs.error(err);
            }
        });
    }
    /**
     * Metodo que se ejecuta al ocurrir una excepcion al procesar mensajes.
     * @param topic Topico del Error
     * @param data Mensaje de la peticion
     * @param error Error que se ejecuta
     */
    exception(topic, data, error) {
        // Generar el transactionId
        const transactionId = (new Date().getTime());
        // Enviar el Error a kafka
        this.sendError(topic, transactionId, error);
        // Retornar el error al consumidor
        this.response({
            data: null,
            metadata: {
                id_transaction: `error_${transactionId}`,
                owner: this.Topic,
            },
        }, -99);
    }
    /**
     * Metodo que se ejecuta en el hellcheck
     * @publick
     */
    HellCheck() {
        return true;
    }
    /**
     * Metodo para generar una excepcion en el codigo
     * @param code Codigo de error
     */
    emitError(code) {
        // Recuperar el texto del error
        const text = this.getError(code);
        // Generar la instancia del error
        return new Error_1.MsError(text, code);
    }
    /**
     * Metodo para iniciar el HellCheck HTTP
     * @private
     */
    startHttpHellCheck() {
        // Crear la instancia del servidor
        const server = http.createServer((request, response) => {
            // Validar
            try {
                if (this.HellCheck()) {
                    response.end('ok');
                }
                else {
                    // Cambiar la cabecera
                    response.statusCode = 500;
                    response.end('ERROR');
                }
            }
            catch (error) {
                // Cambiar la cabecera
                response.statusCode = 500;
                response.end('ERROR');
            }
        });
        // Iniciar el server
        server.listen(80, () => {
            // Inidicar que todo esta bien
            this.logs.info('[HellCheck] listening on 0.0.0.0:80');
        });
    }
    /**
     * Metodo para recuperar un Error.
     * @param code Codigo de Error
     */
    getError(code) {
        return this.ErrorColl.get(code);
    }
    /**
     * Metodo para recuperar la configuracion de un elemento
     * registrado con los decoradores.
     * @param Name Nombre de la configuracion
     */
    getConfHandler(Name) {
        // Validar si existe la funcion que se solicita
        if (Reflect.hasMetadata(Name, this)) {
            // Regresar la configuracion
            return Reflect.getMetadata(Name, this);
        }
        else if (Reflect.hasMetadata('definition:listener', this)) { // Validar si existe listener
            // Regresar el generico
            return Reflect.getMetadata('definition:listener', this);
        }
        else {
            // No existe ningun elemento que responda a la peticion.
            return null;
        }
    }
}
exports.Microservice = Microservice;
