"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const EventEmitter = require("events");
const kafka_node_1 = require("kafka-node");
const util_1 = require("util");
const logs_1 = require("./logs");
const util_2 = require("./util");
/**
 *  Clase para las conexiones y publicaciones sobre
 *  la cola de mensajes.
 */
class Kafka extends util_2.default {
    /**
     * Contructor de la clase.
     * @param kafkaHost Servidores Kafka
     * @param Topic Topico de escucha
     */
    constructor(Conf) {
        //  Contructor padre
        super();
        /**
         * Manejador de eventos
         * @protected
         */
        this.event = new EventEmitter();
        /**
         * Manejador de logs
         * @protected
         */
        this.logs = logs_1.default;
        // Asignar la configuracion
        this.conf = Conf;
        this.conf.ClearQueque = util_1.isUndefined(Conf.ClearQueque) ? true : Conf.ClearQueque;
        /**
         * Definir topico con soporte para servicios con nomemclatura
         * legada.
         */
        // Ver si es legacy
        if (this.conf.Legacy) {
            // Es un servicio legado
            this.Topic = this.conf.Name;
        }
        else {
            // Crear la combinacion
            this.Topic = `${this.conf.App}_${this.conf.Version}_${this.conf.Name}`;
        }
        // Verificar si se paso la cadena de conexion a kafka
        if (this.conf.Kafka) {
            // Asignar la pasada como configuracion
            this.kafkaHost = this.conf.Kafka;
        }
        else {
            // Verificar que se asigno una por variable de ambiente
            if (process.env.KAFKA) {
                // Asignar
                this.kafkaHost = process.env.KAFKA.split(',');
            }
            else {
                this.logs.error('No se proporciono la configuracion de KAFKA');
                process.exit(-1); // Finalizar el programa
            }
        }
        // Crear el cliente de kafka.
        this.client = new kafka_node_1.KafkaClient({ kafkaHost: this.kafkaHost.join(',') });
        // Registrar el producer.
        this.producer = new kafka_node_1.Producer(this.client);
        // Eventos del producer
        this.producer.on('ready', () => {
            // Opciones del consumidor
            const options = {
                groupId: this.Topic,
                kafkaHost: this.kafkaHost.join(','),
            };
            // Crear el consumidor
            this.consumer = new kafka_node_1.ConsumerGroup(options, [this.Topic]);
            // Escuchar cuando se realice la conexion
            this.consumer.on('connect', () => {
                const text = `Escuchando el topico ${this.Topic} en el Kafka ${this.kafkaHost.join(',')}`;
                // Notificar que la conexion fue exitosa.
                this.logs.info(text);
            });
            // Llegada de mensajes
            this.consumer.on('message', (message) => {
                try {
                    // Convertir el mensaje
                    const data = JSON.parse(Buffer.from(message.value.toString(), 'base64').toString('utf8'));
                    // Pasar el mensaje al que lo procesara
                    this.message(data);
                }
                catch (error) {
                    // Disparamos la excepcion
                    this.exception(this.Topic, message.value.toString(), error);
                }
            });
            // Errores
            this.consumer.on('error', (err) => {
                // Enviar el error
                this.logs.error(err);
            });
        });
        this.producer.on('error', (err) => {
            // Enviar el error
            this.logs.error(err);
        });
    }
}
exports.default = Kafka;
